package TSim;

/** Semaphore shared with other trains on the same track. */
public class Semaphore {
    private int id;
    private TSimInterface tsi;

    Semaphore(int id, int s, TSimInterface tsi) {
	this.id=id;
	this.tsi=tsi;
	sendOp("CreateSemaphore "+s+" ");
    }

    private synchronized boolean sendOp(String opName) {
	TrainEvent tE = tsi.lastEvent;
	
	tsi.sTSim.write(opName + id);
	
	CommandStatus cStat = (CommandStatus) tsi.commandFIFO.get();

	return (cStat.getStatus() == CommandStatus.OK);
    }

    /** Acquire semaphore (P operation). */
    public void acquire() {
	sendOp("AcquireSemaphore ");
    }

    /** Release semaphore (V operation). */
    public void release() {
	sendOp("ReleaseSemaphore ");
    }

    /** Acquire semaphore if free. 
     * @return Whether the semaphore has acquired.
     */
    public boolean tryAcquire() {
	return sendOp("TrySemaphore ");
    }
}
