package TSim;

/** An (erroneous) event for a train.
 *
 */
public class TrainEvent implements TSimInformation 
{
    /** Event type: Train has collided with another train. */
    public static final int TRAIN_COLLISION = 0x01;
    /** Event type: Train has collided with a stop at the end of the track. */
    public static final int STOP_COLLISION  = 0x02;
    /** Event type: Train has run off the rails. */
    public static final int DERAILMENT      = 0x03;
    
    int event;
   
    /** Create a new TrainEvent for the specified train id and with 
     *  the specified event type.
     *
     *  @param event     the type of the event.
     *
     */
    public TrainEvent(int event)
    {
	this.event = event;
    }
    
    /** Returns the type of the event.
     *
     *  @return the type of the event.
     *
     */
    public int getEvent()
    {
	return event;
    }

    /** Returns a string representation of the event. This is used when 
     *  creating exception messages.
     *
     *  @return the string representation of the event.
     *
     */
    
    public String toString() 
    {
	String eventString = "unknown event";
	
	switch (event) {
	case TRAIN_COLLISION :
	    eventString = " train collision";
	    break;
	case STOP_COLLISION :
	    eventString = " stop collision";
	    break;
	case DERAILMENT :
	    eventString = " derailment";
	    break;
	}
	
	return "Fatal error for train: " + eventString;
    }
}
