#include <stdlib.h>
#include <X11/Intrinsic.h>
#include "train.h"
#include "trainP.h"
#include "semaphores.h"
#include "resources.h"

void Acknowledge(tTrain *train, tStatus status);

static tStatus EnsureValid(Semaphore **p, Semaphore **s, Boolean create) {
    if (*s==NULL) {
	if (create) {
	    *s=NEW(Semaphore);
	    *p=*s;
	    (*s)->value=-1;
	    (*s)->next=NULL;
	} else
	    return IllegalSemaphore;
    }
    return SuccessStatus;
}

static tStatus GetSemaphore(tTrainLine train_line, int id, Semaphore **s,
			    Boolean create) {
    int i;
    tStatus stat;
    Semaphore **p=&(train_line->semaphores);
    *s=train_line->semaphores;
    if ((stat=EnsureValid(p, s, create))) return stat;

    for(i=0;i<id;i++) {
	p=&((*p)->next);
	*s=(*s)->next;
	if ((stat=EnsureValid(p, s, create))) return stat;
    }
    return SuccessStatus;
}

tStatus CreateSemaphore(tTrainLine train_line, int id, int val) {
    Semaphore *s;
    tStatus stat;

    if ((stat=GetSemaphore(train_line, id, &s, 1))) return stat;
    if (s->value<0)
	s->value=val;
    return SuccessStatus;
}

tStatus AcquireSemaphore(tTrain *train, int id) {
    Semaphore *s;
    tStatus stat;
    tTrainLine train_line=train->line;

    if ((stat=GetSemaphore(train_line, id, &s, 0))) return stat;
    if (s->value>0) {
	s->value--;
	if (use_log())
	    fprintf(stderr, "%d: acquired semaphore and decremented to %d\n",
		    train->no, s->value);
	return SuccessStatus;
    } else {
	train->waiting_semaphore=id;
	if (use_log())
	    fprintf(stderr, "%d: Waiting on semaphore\n", train->no);

	return NoStatus;
    }
}

tStatus TrySemaphore(tTrainLine train_line, int id) {
    Semaphore *s;
    tStatus stat;

    if ((stat=GetSemaphore(train_line, id, &s, 0))) return stat;
    if (s->value>0) {
	s->value--;
	if (use_log())
	    fprintf(stderr, "acquired semaphore and decremented to %d\n",
		    s->value);
	return SuccessStatus;
    } else {
	if (use_log())
	    fprintf(stderr, "failed to acquire semaphore\n");
	return SemaphoreNotFree;
    }
}

tStatus ReleaseSemaphore(tTrainLine train_line, int id) {
    Semaphore *s;
    tStatus stat;
    tTrain *t;

    if ((stat=GetSemaphore(train_line, id, &s, 0))) return stat;

    t=train_line->trains;
    while((t!=NULL)&&(t->waiting_semaphore!=id))
	t=t->next;

    if (t!=NULL) {
	if (use_log())
	    fprintf(stderr, "released semaphore and acquired by %d\n", t->no);
	Acknowledge(t, SuccessStatus);
	t->waiting_semaphore=-1;
    } else {
	s->value++;
	if (use_log())
	    fprintf(stderr, "released semaphore and incremented to %d\n", s->value);
    }

    return SuccessStatus;
}
