#include <math.h>
#include <stdio.h>
#include <X11/Xlib.h>			/* for Xlib definitions */
#include <X11/StringDefs.h>		/* for useful atom names */
#include <X11/Intrinsic.h>
#include <sys/wait.h>

#include "train.h"
#include "trainP.h"
#include "resources.h"

static int elapsedtime=0;
static int lastsecond=-1;

/* prototype for gcc */
void AddTimeOut(tTrainLine train_line);
void SendTrainEvent(tTrain *train, tEvent event);
Boolean IsCovered(tRail *rail, tRailPos *bitmap_pos);
void Acknowledge(tTrain *train, tStatus status);

int fsign(f)
     float f;
{
  if(f > 0.0)
    return(1);
  else if(f < 0.0)
    return(-1);
  else
    return(0);
}

void MoveTrains(train_line, dummy)
     tTrainLine train_line;
{
  tTrain *train;
  float speed_diff = timeout_interval() * acceleration();
  tEvent event;
  int xo, yo, hit;
  tTrainPart *car;
  tRailPos rp, st;

  while(wait3(NULL, WNOHANG, NULL)>0);

  if (XtAppGetExitFlag(XtWidgetToApplicationContext(train_line->top))) {      
      AddTimeOut(train_line);
      return;
  }

  for(train = train_line->trains; train; train = train->next) {
    if(train->speed != train->wanted_speed) {
      train->speed += (fsign(train->wanted_speed - train->speed) * speed_diff);
      if(fabs(train->speed - train->wanted_speed) < speed_diff)
	train->speed = train->wanted_speed;
    }
    train->offset_frac += timeout_interval() * fabs(train->speed);
    if(train->offset_frac >= 1.0) {
      int int_offset = train->offset_frac;
      train->offset_frac -= int_offset;
      if((event = 
	 MoveTrain(train_line, train->no, fsign(train->speed) * int_offset))) {
	/* Collision, stop train! */
	train->speed = 0.0;
	train->wanted_speed = 0.0;
	SendTrainEvent(train, event);
      }
    }
    if ((train->speed == 0.0)&&train->loading) {
	hit=0;
	for(car = train->parts; car; car = car->next)
	    for(xo=car->pos.x-train->load_distance;
		xo<car->pos.x+train->load_distance; xo++)
		for(yo=car->pos.y-train->load_distance;
		    yo<car->pos.y+train->load_distance; yo++)
		    if ((xo>=0)&&(yo>=0)&&(xo<train_line->space.x)&&
			(yo<train_line->space.y)&&
			(train_line->rails[xo][yo].custom)) {
			hit=1;
			break;
		    }
	
	if (!hit) {
	    train->loading=0;
	    Acknowledge(train, TrainNotAtStation);
	} else {
	    if (IsCovered(&train_line->rails[xo][yo], &rp)) {
		st=rp;
	    } else
		st=Pos(xo, yo);
	    if ((train->last_station.x!=st.x)||(train->last_station.y!=st.y)) {
		train->last_station=st;
		train->ticks_since_last_load=0;
	    }
	}
	train->loading--;
	if (!train->loading)
	    Acknowledge(train, SuccessStatus);
    } else {
	train->ticks_since_last_load++;
	if (strict()&&(train->ticks_since_last_load>
		       timeout()/timeout_interval()))
	    SendTrainEvent(train, TimeoutEvent);
    }
  }
  elapsedtime++;
  if (use_log()) {
      int sec=(int)(((float)elapsedtime)*timeout_interval());
      if (sec!=lastsecond)
	  fprintf(stderr, "%d seconds elapsed...\n", sec);
      lastsecond=sec;
  }
  AddTimeOut(train_line);
}
      
void AddTimeOut(train_line)
     tTrainLine train_line;
{
    XtAppAddTimeOut(XtWidgetToApplicationContext(train_line->top), 
		    (long)(timeout_interval() * 1000.0 / speedup_factor()),
		    MoveTrains, (XtPointer)train_line);
}
