/* tsim magnus@cs.chalmers.se 1990 */
/* tsim for Linux, algo@chl.chalmers.se 2000 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>

#include "Sprite.h"
#include "train.h"
#include "file.h"
#include "resources.h"
#include "signal.h"

#ifdef SOLARIS
#include <stdlib.h>
#endif

#ifdef LINUX
#include <stdlib.h>
#endif
#include <time.h>

/* prototypes for gcc */
extern void AddListenSocket (XtAppContext apc, tTrainLine train_line);
extern Widget CreateDrawingPalette (String name, Widget parent, 
				    tTrainLine train_line);
extern void StopNetworkTraffic (tTrainLine train_line);

static String fallback_resources[] = {
  "tsim*illegalNumber.dialog.label: Illegal number or direction missing!",
  "tsim*cannotCreateTrain.dialog.label: Cannot create train!",
  "tsim*noOfCarsAndDirection.dialog.label: Enter number of cars and direction",
  "tsim*noOfCarsAndDirection.dialog.value: 1 Left",
  "tsim*cannotSave.dialog.label: Cannot save train line!",
  "tsim*save.dialog.label: Save train line in:",
  "tsim*save.dialog.value: ",
  "tsim*speed.dialog.label: Enter speed",
  "tsim*speed.dialog.value: 20",
  "tsim*illegalSpeed.dialog.label: Illegal speed!",
  "tsim*bulldozerButton.label: Delete",
  "tsim*speedButton.label: Speed",
  "tsim*switchButton.label: Switch",
  "tsim*quit.label: Quit",
  "tsim*save.label: Save",
  "tsim*tools.geometry: 467x64",
  "tsim*coordinates.label: v0.81-HUT", /* version */
  NULL
  };

static XtAppContext apc;

void Terminate (int i) {
//    XtAppSetExitFlag(apc);
    exit(i);
}

XrmOptionDescRec Desc[] =
{
    {"-log", "*log", XrmoptionNoArg, "yes"},
    {"-strict", "*strict", XrmoptionNoArg, "yes"},
    {"-bitmapDir", "*bitmapDir", XrmoptionSepArg, NULL},
    {"-soundDir", "*soundDir", XrmoptionSepArg, NULL},
    {"-timeoutInterval", "*timeoutInterval", XrmoptionSepArg, NULL},
    {"-timeOut", "*timeOut", XrmoptionSepArg, NULL},
    {"-acceleration", "*acceleration", XrmoptionSepArg, NULL},
    {"-playCmd", "*playCmd", XrmoptionSepArg, NULL},
    {"-speedupFactor", "*speedupFactor", XrmoptionSepArg, NULL}
};

int main(int argc, char **argv) {
    int xargc;
    char **xargv;
    Widget toplevel, dp;
    tTrainLine tl;

#ifdef SOLARIS
    srand((int) time ((time_t *) 0));
#else
    /* srand is available in Linux, but srandom is better */
    srandom ((int) time (NULL));
#endif

#if 0
    (void)XSynchronize(XtDisplay(toplevel),1);
#endif

    xargc = argc;
    xargv = (char **) XtMalloc (argc * sizeof (char *));
    memcpy(xargv, argv, argc * sizeof (char *));
    toplevel = XtAppInitialize(&apc, "Tsim", Desc, XtNumber (Desc), &xargc,
			       xargv, fallback_resources, NULL, 0);
    GetResources(toplevel);

    if (xargc == 2) {
      if(!(tl = LoadTrainLine(xargv[1], toplevel)))
	exit(1);
    } else {
      int x, y;
      if(xargc <= 1 || !sscanf(xargv[1], "%d", &x) ||
	 !sscanf(xargv[2], "%d", &y)) {
	  x = 20;
	  y = 20;
      }
      tl = CreateTrainLine(toplevel, Pos(x,y));
    }

    free(xargv);

    AddListenSocket(apc, tl);

    dp = (Widget)CreateDrawingPalette("tools", toplevel, tl);

    XtRealizeWidget (toplevel);

    signal(SIGTERM, Terminate);
    signal(SIGINT, Terminate);

    XtAppMainLoop (apc);

    StopNetworkTraffic (tl);

    return 0;
}
